/*
 * Steps tu uses this application.
 * Turn on NXT.
 * Run this application and click connect.
 * Wait until connection is active.
 * Run NXT application (FeaturesTest1.nxc).
 * 
 */
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace NXTRemoteCommander
{
    public partial class MainForm : Form
    {
        /// <summary>
        /// List of commands to send to application that is running on NXT.
        /// </summary>
        enum NXTCommands:byte { 
            GoForward = (byte)'f',
            GoBack    = (byte)'b',
            TurnRight = (byte)'r',
            TurnLeft  = (byte)'l'
        };
        
        /// <summary>
        /// Form constructor.
        /// </summary>
        public MainForm()
        {
            InitializeComponent();
        }

        /// <summary>
        /// Activate/deactivate connection to NXT via BlueTooth using serial port.
        /// </summary>
        private void btnConnection_Click(object sender, EventArgs e)
        {
            // Disable the button while application process this command.
            btnConnection.Enabled = false;

            // If connection is already open close it, disable command buttons and update this button text,
            // else open connection.
            if (NXTconn.IsOpen)
            {
                grpCommands.Enabled = false;
                NXTconn.Close();
                btnConnection.Text = "Connect";
            }
            else
            {
                // Test if user set valid com port name.
                int result;
                bool num = int.TryParse(txtPort.Text.Substring(3), out result);
                if (txtPort.Text.ToUpper().IndexOf("COM") == 0 && num && result >= 0)
                {
                    // Try to connect to NXT. If BricxCC is connected to NXT via Bluetooth too this try will fails!
                    NXTconn.PortName = txtPort.Text;
                    try
                    {
                        NXTconn.Open();
                    }
                    catch (Exception err)
                    {
                        MessageBox.Show(err.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                    if (NXTconn.IsOpen)
                    {
                        // Enables command buttons and update this button text.
                        grpCommands.Enabled = true;
                        NXTconn.ReadTimeout = 1500;
                        btnConnection.Text = "Disconnect";
                    }
                }
                else
                    MessageBox.Show("Wrong Port Name!.", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            
            // Enable it back.
            btnConnection.Enabled = true;
        }

        /// <summary>
        /// Turn off connection to NXT, if it's opened, when user close the application.
        /// </summary>
        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (NXTconn.IsOpen)
                NXTconn.Close();
        }

        /// <summary>
        /// Send command to NXT application.
        /// </summary>
        /// <param name="c">The command to send.</param>
        private void SendCommand(byte c)
        {
            // Message array structure.
            byte[] message = { 0x00,        // Direct command telegram, response required.
                               0x09,        // Command: MESSAGEWRITE.
                               0x00,        // Inbox number (0 - 9).
                               0x02,        // Message Size: 2 bytes. Message must be a null terminated string.
                               0x00,        // Message: command to send, will be parameter c.
                               0x00         // Null terminator for the message string.
            };
            message[4] = c;
            byte[] response = new byte[3];  // Will be: 0x02, 0x09, Status Byte (0 success, greater than 0 is error code =>
                                            // 0xEC is not active program).

            // Header of message to sent.
            byte[] bluetoothHeader = { 0x00, 0x00 };    // LSB and MSB (always 0 in this application).
            bluetoothHeader[0] = (byte) message.Length;

            // Try to send the command.
            // TODO: test the response.
            try
            {
                NXTconn.Write(bluetoothHeader, 0, bluetoothHeader.Length);
                NXTconn.Write(message, 0, message.Length);
                int length = NXTconn.ReadByte() + 256 * NXTconn.ReadByte();     // There is an error if not equal to 3!
                for (int i = 0; i < length; i++)
                    response[i] = (byte) NXTconn.ReadByte();
            }
            catch (Exception err)
            {
                MessageBox.Show(err.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        /// <summary>
        /// Sends GoForward command to NXT application.
        /// </summary>
        private void btnGoForward_Click(object sender, EventArgs e)
        {
            SendCommand((byte)NXTCommands.GoForward);
        }

        /// <summary>
        /// Sends GoBack command to NXT application.
        /// </summary>
        private void btnGoBack_Click(object sender, EventArgs e)
        {
            SendCommand((byte)NXTCommands.GoBack);
        }

        /// <summary>
        /// Sends TurnLeft command to NXT application.
        /// </summary>
        private void btnTurnLeft_Click(object sender, EventArgs e)
        {
            SendCommand((byte)NXTCommands.TurnLeft);
        }

        /// <summary>
        /// Sends TurnRight command to NXT application.
        /// </summary>
        private void btnTurnRight_Click(object sender, EventArgs e)
        {
            SendCommand((byte)NXTCommands.TurnRight);
        }
    }
}